// -*- C++ -*-
/*!
 * @file  LINETRACE_CTRL.cpp
 * @brief Line trace controll
 * @date $Date$
 *
 * $Id$
 */

#include "LINETRACE_CTRL.h"

// Module specification
// <rtc-template block="module_spec">
static const char* linetrace_ctrl_spec[] =
  {
    "implementation_id", "LINETRACE_CTRL",
    "type_name",         "LINETRACE_CTRL",
    "description",       "Line trace controll",
    "version",           "0.1",
    "vendor",            "Vstone",
    "category",          "Test",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "0",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.CPUMode", "0",
    "conf.default.MotorGainL", "150",
    "conf.default.MotorGainR", "150",
    "conf.default.SensorThL", "500",
    "conf.default.SensorThR", "500",
    "conf.default.Speed", "60",
    // Widget
    "conf.__widget__.CPUMode", "text",
    "conf.__widget__.MotorGainL", "text",
    "conf.__widget__.MotorGainR", "text",
    "conf.__widget__.SensorThL", "text",
    "conf.__widget__.SensorThR", "text",
    "conf.__widget__.Speed", "text",
    // Constraints
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
LINETRACE_CTRL::LINETRACE_CTRL(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_Sensor1In("Sensor1", m_Sensor1),
    m_Sensor2In("Sensor2", m_Sensor2),
    m_Gain1Out("Gain1", m_Gain1),
    m_Gain2Out("Gain2", m_Gain2),
    m_Motor1Out("Motor1", m_Motor1),
    m_Motor2Out("Motor2", m_Motor2),
    m_ModeOut("Mode", m_Mode)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
LINETRACE_CTRL::~LINETRACE_CTRL()
{
}



RTC::ReturnCode_t LINETRACE_CTRL::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("Sensor1", m_Sensor1In);
  addInPort("Sensor2", m_Sensor2In);
  
  // Set OutPort buffer
  addOutPort("Gain1", m_Gain1Out);
  addOutPort("Gain2", m_Gain2Out);
  addOutPort("Motor1", m_Motor1Out);
  addOutPort("Motor2", m_Motor2Out);
  addOutPort("Mode", m_ModeOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("CPUMode", m_CPUMode, "0");
  bindParameter("MotorGainL", m_MotorGainL, "150");
  bindParameter("MotorGainR", m_MotorGainR, "150");
  bindParameter("SensorThL", m_SensorThL, "500");
  bindParameter("SensorThR", m_SensorThR, "500");
  bindParameter("Speed", m_Speed, "60");
  
  // </rtc-template>
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t LINETRACE_CTRL::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t LINETRACE_CTRL::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t LINETRACE_CTRL::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t LINETRACE_CTRL::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t LINETRACE_CTRL::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t LINETRACE_CTRL::onExecute(RTC::UniqueId ec_id)
{
	//[h̎擾Əo
	if(m_Mode.data!=m_CPUMode){
		m_Mode.data = m_CPUMode;
		m_ModeOut.write();
		printf("Mode=%d\n",m_Mode.data);
	}

	//[^QC̎擾Əo
	if(m_Gain1.data!=m_MotorGainL){
		m_Gain1.data = m_MotorGainL;
		m_Gain1Out.write();
		printf("Gain1=%d\t",m_Gain1.data);
	}

	if(m_Gain2.data!=m_MotorGainR){
		m_Gain2.data = m_MotorGainR;
		m_Gain2Out.write();
		printf("Gain2=%d\n",m_Gain2.data);
	}


	//ZT̎擾
	if(m_Sensor1In.isNew()) m_Sensor1In.read();
	if(m_Sensor2In.isNew()) m_Sensor2In.read();
	printf("Sensor1=%d , Sensor2=%d\n",m_Sensor1.data,m_Sensor2.data);

	//[^x̌
	//uivƂāAZTCA̎ԗւ~
	{
		static short pre_spd_l=0,pre_spd_r=0;
		m_Motor1.data = m_Speed;
		m_Motor2.data = -m_Speed;

		//ZTCꍇ
		if(m_Sensor2.data>m_SensorThL) m_Motor1.data = 0;
		//EZTCꍇ
		if(m_Sensor1.data>m_SensorThR) m_Motor2.data = 0;

		printf("Spd1=%d , Spd2=%d\n",m_Motor1.data,m_Motor2.data);
		//[^xCPU{[h䑤RTR|[lg֏o
		if(pre_spd_l!=m_Motor1.data){
			m_Motor1Out.write();
			pre_spd_l=m_Motor1.data;
		}
		if(pre_spd_r!=m_Motor2.data){
			m_Motor2Out.write();
			pre_spd_r=m_Motor2.data;
		}
	}
	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t LINETRACE_CTRL::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t LINETRACE_CTRL::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t LINETRACE_CTRL::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t LINETRACE_CTRL::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t LINETRACE_CTRL::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void LINETRACE_CTRLInit(RTC::Manager* manager)
  {
    coil::Properties profile(linetrace_ctrl_spec);
    manager->registerFactory(profile,
                             RTC::Create<LINETRACE_CTRL>,
                             RTC::Delete<LINETRACE_CTRL>);
  }
  
};


